import BindingScope from "../bindings/binding_scope";
import BindingType from "../bindings/binding_type";
import * as ERROR_MSGS from "../constants/error_msgs";
import { getServiceIdentifierAsString } from "../utils/serialization";
import resolveInstance from "./instantiation";
function _resolveRequest(request) {
    var bindings = request.bindings;
    var childRequests = request.childRequests;
    var targetIsAnAray = request.target && request.target.isArray();
    var targetParentIsNotAnArray = !request.parentRequest ||
        !request.parentRequest.target ||
        !request.parentRequest.target.matchesArray(request.target.serviceIdentifier);
    if (targetIsAnAray && targetParentIsNotAnArray) {
        return childRequests.map(function (childRequest) {
            return _resolveRequest(childRequest);
        });
    }
    else {
        var result = null;
        var binding = bindings[0];
        var isSingleton = binding.scope === BindingScope.Singleton;
        if (isSingleton && binding.activated === true) {
            return binding.cache;
        }
        switch (binding.type) {
            case BindingType.ConstantValue:
                result = binding.cache;
                break;
            case BindingType.DynamicValue:
                result = binding.dynamicValue(request.parentContext);
                break;
            case BindingType.Constructor:
                result = binding.implementationType;
                break;
            case BindingType.Factory:
                result = binding.factory(request.parentContext);
                break;
            case BindingType.Function:
                result = binding.cache;
                break;
            case BindingType.Provider:
                result = binding.provider(request.parentContext);
                break;
            case BindingType.Instance:
                result = resolveInstance(binding.implementationType, childRequests, _resolveRequest);
                break;
            case BindingType.Invalid:
            default:
                var serviceIdentifier = getServiceIdentifierAsString(request.serviceIdentifier);
                throw new Error(ERROR_MSGS.INVALID_BINDING_TYPE + " " + serviceIdentifier);
        }
        if (typeof binding.onActivation === "function") {
            result = binding.onActivation(request.parentContext, result);
        }
        if (isSingleton) {
            binding.cache = result;
            binding.activated = true;
        }
        return result;
    }
}
function resolve(context) {
    return _resolveRequest(context.plan.rootRequest);
}
export default resolve;
